#!/usr/bin/python


import os
import sys
import re
import urllib

cvs_rep="."
wiki_rep="wiki"
wiki_url="http://gpl.insa-lyon.fr/GplWiki/"
wiki_header = "#pragma section-numbers off\n[ AutonomyProject ] - [ AutonomyManuscrit ] - [ Chapter%sEn ]\n"


latex2wikire = \
         [[r"\\og\{?\}?~?\s?", r"<< "],              #<< >>
	  [r"\s?~?\\fg\{?\}?", r" >>"], 
	  [r"\n +", r"\n"],                          #espaces en debut de ligne
	  [r".*\\separateur.*", r"----"],            #separateur
	  [r"\\oe ", r"oe"],                         #oe
	  [r"\\\\", "\n"],                           #retour chariot
#	  [r"%\$Revision:(.*?)\$.*\$Date:(.*?)\$", r"== Revision CVS \1 datee du \2 =="],
          [r"\$:\$", r":"],                         #: pour dates
	  [r"\$\^\{(.*?)\}\$", r"^\1^"],             #exposant
	  [r"\$\^(.)\$", r"^\1^"],                   #exposant
	  [r"\$_\{(.*?)\}\$", r"^\1^"],              #exposant
	  [r"\$_(.)\$", r"^\1^"],                    #exposant
	  [r"\$(.*?)\$", r"'''''\1'''''"],           #formules
	  [r"\\texttt\{(.*?)\}", r"<tt>\1</tt>"],    #texttt
	  [r"\\%", "<<POURCENT>>"],                  #symbole "%"
	  [r"%TODO:(.*)", r" {X} '''\1'''"],         #Notes TODO
	  [r"%%%.*\n", ""],                            #commentaires ajoutes par AUC-TeX
	  [r"%(.*)", r" {i} '''\1'''"],              #Note d'information
	  [r"<<POURCENT>>", "%"],                    #symbole "%"
	  [r"\\dater\s*\{(.*?)\}\s*\{(.*?)\}\s*\{(.*?)\}", "{{{\n\\1\n\\2\n\\3\n}}}"],
	                                             #metadate
	  [r"(?s)\\citer\s*\{(.*?)\}\s*\{(.*?)\}", r"{1} '''\1\n\t\t--- \2'''"],
	                                             #citation
	  [r"\\chapter\{(.*?)\}", r"= \1 ="],
                                                     #chapitre
	  [r"(?s)\\dire\{(.*?)\}", r"''\1''"],       #dire
	  [r"(?s)\\penser\s*\{(.*?)\}", r"'''\1'''"],#penser
	  [r"([a-zA-Z])([:!;?])", r"\1 \2"],                     #espace avant les signes de ponctuation
	  [r"(?s)\\begin\{verbatim\}(.*?)\\end\{verbatim\}", r"{{{\n\1}}}"]
	                                             #environnement VERBATIM
#          [r"\\begin\{itemize\}", "{2}"],
#          [r"\\end\{itemize\}", "{3}"],
#          [r"\\begin\{itemize\}\n", ""],
#          [r"\\end\{itemize\}\n", ""],
#          [r"\\item (.*)", r"* \1"],
#          [r"\\vspace\{1cm\}", r"{4}"],
#      	  [r"^\\vspace\{1cm\}\n\\texttt\{\n", r"{{{\n"],
#	  [r"^\}\n\\vspace\{1cm\}\n", r"}}}\n"]
	  ];


wiki2latexre = \
         [[r"\{\{\{\s*(.*)\n(.*)\n(.*)\s*\}\}\}", r"\\dater{\1}\n{\2}\n{\3}"],
	  [r"\{\{\{\s*(.*)\n(.*)\s*\}\}\}", r"\\dater{\1}\n{\2}{}"],
	  [r"(?s)\{\{\{(.*?)\}\}\}", r"\\begin{verbatim}\n\1\\end{verbatim}"],
          [r"#pragma.*\n", r""],                     #en-tete
          [r"{i} '''''Revision.*?'''''", r""],       #Version CVS
	  [r"\n +", r"\n"],                          #espaces en debut de ligne
       	  ["\r", ""],                                #Retours chariot DOS
	  [r" ([:!;?])", r"\1"],
	  [r"%", r"\\%"],
	  [r" ?{X}\s+'''(.*?)'''", r"%TODO:\1"],
	  [r" ?{i}\s+'''(.*?)'''", r"%\1"],
          [r"([0-9]):([0-9])", r"\1$:$\2"],          #Espaces autour des :
	  [r"\^(.*?)\^", r"$_{\1}$"],
	  [r"----", r"\\separateur"],
	  [r"^\s*\*\*\*\s*$", r"\\separateur"],
	  [r".*\* \* \*.*", r"\\separateur"],
	  [r"\\oe\s", r"oe"],
	  [r"oe", r"\oe "],
	  [r"c\oe ffi", r"coeffi"],
	  [r"m\oe lle", r"moelle"],
	  [r"c\oe xis", r"coexis"],
	  [r"<<\s*", r"\\og{}~"],
	  [r"\s*>>", r"~\\fg{}"],
	  [r"== Revision CVS\s*([0-9.]+).*?(20.*)\s+==", r"%$Revision: \1 $ $Date: \2$"],
	  [r"= (.*) =", r"\\chapter{\1}"],
	  [r"'''''(.*?)'''''", r"$\1$"],
	  [r"(?s)\{1\}\s*'''(.*?)\s*---\s*(.*?)'''", r"\\citer{\1}\n{\2}"],
	  [r"(?s)'''(.*?)'''", r"\\penser{\1}"],
	  [r"(?s)''(.*?)''", r"\\dire{\1}"],
	  [r"^\s*--*\s*", r"--- "],
	  [r"^\s*\\dire\{", r"--- \dire{"],
	  [r"\[\s*AutonomyProject.*\n\n?", ""],
	  [r"<tt>(.*?)</tt>", r"\texttt{\1}"]
#          [r"\n\s*\*\s+(.*)", r"\item \1"],
#	  [r"^\{\{\{\n", r"\vspace{1cm}\n\texttt{\n"],
#	  [r"^\}\}\}\n", r"}\n\vspace{1cm}\n"],
#          [r"((\\item.*\n)+)", r"\begin{itemize}\n\1\end{itemize}\n"]
#          [r"\{2\}", r"\begin{itemize}"],
#          [r"\{3\}", r"\end{itemize}"]          
	  ];    

orig2wikire = \
            [[r"^[\n\s]*?\"(.*?)\"[\n\s]*?--(.*)", r"{1} '''\1'''\n---\2"],
             [r"\n", r"\n\n"],
             [r"\n\"", r"\n--- ''"],
             [r"\"", "''"]
             ];


def download_page(num):
    "download a single page"
    print("Downloading chapter %s" % num);
    fd = urllib.urlopen(wiki_url+"Chapter%sFr?action=raw" % num);
    page = fd.read();
    fd.close();
#    page = unicode(page, 'utf-8').encode('iso-8859-1');
    try:
        page = unicode(page, 'utf-8').encode('iso-8859-1');
    except:
	print("UNICODE ERROR");
        page = unicode(page, 'utf-8').encode('iso-8859-1', 'replace')



    fd = open( os.path.join(cvs_rep, "chap%s.wiki" % num), "w");
    fd.write(page);
    fd.close();


def wiki_to_latex(num):
    "convert a file to latex"
    print("Converting chapter %s to LaTeX" % num);
    fd = open( os.path.join(cvs_rep, "chap%s.wiki" % num), "r");
    page = fd.read();
    fd.close();
    for regexp in wiki2latexre:
        page=re.sub( regexp[0], regexp[1], page);
    fd = open( os.path.join(cvs_rep, "chap%s.tex" % num), "w");
    fd.write(page);
    fd.close();

def latex_to_wiki(num):
    "convert a file to wiki"
    print("Converting chapter %s to Wiki" % num);
    fd = open( os.path.join(cvs_rep, "chap%s.tex" % num), "r");
    page = fd.read();
    fd.close();
    page = wiki_header % num + page;
    for regexp in latex2wikire:
        page=re.sub( regexp[0], regexp[1], page);
    fd = open( os.path.join(cvs_rep, "chap%s.wiki" % num), "w");
    fd.write(page);
    fd.close();


def upload_page(num):
    "upload a wiki page"
    print("Uploading chapter %s" % num);
    fd = open( os.path.join(cvs_rep, "chap%s.wiki" % num), "r");
    page=fd.read();
    fd.close();

    forminfo = {};
    forminfo['action']   = 'edit';
    forminfo['comment']  = 'Corrections+a+partir+du+LaTeX';
    forminfo['datestamp']= '0';
    forminfo['savetext'] = unicode(page, 'iso-8859-1').encode('utf-8');
    
    fd = urllib.urlopen(wiki_url+"Chapter%sFr#preview" % num, urllib.urlencode(forminfo));
    fd.read();
    fd.close();
    

#        for iter in re.finditer(r"(?m)^([0-9]+) - (.*?)\n(.*)\n(src|href)\n[0-9]+ -", txt):

def upload_original(txt):
    "Identify chapters from the english text, split them, and upload them one by one"
    prev_num="-1";
    cont =1;
    while cont:
        #Match an entire chapter.
        iter = re.search(r"(?s)\n([0-9]{1,2}) ?- ([^\n]*[^0-9\n])\n(.*?)(\n[0-9]{1,2} - |$)", txt)
        if iter == None:
            print("NEXT CHAPTER NOT FOUND");
            sys.exit(1)
        
        #Extract data from it

        num   = iter.group(1);
        titre = iter.group(2);
        chap  = iter.group(3);
        txt   = txt[iter.end(3):];
        cont  = len(iter.group(4));

        if int(num) != int(prev_num)+1:
            print("ERROR : chapter count not contiguous " + num);
            sys.exit(1)
        prev_num = num;
        if len(num) == 1:
            num = "0" + num;

        print( "Extracted chapter %s: <%s>" % (num, titre));


        for regexp in orig2wikire:
            chap=re.sub( regexp[0], regexp[1], chap);

        chap = "#pragma section-numbers off\n[ AutonomyProject ] - [ AutonomyManuscrit ] - [ Chapter%sFr ]\n= %s =\n\n%s" % (num, titre, chap);

        print("Uploading chapter %s" % num);
        forminfo = {};
        forminfo['action']   = 'savepage';
        forminfo['comment']  = 'Generation automatique a partir de l\'original';
        forminfo['datestamp']= '0';
        forminfo['savetext'] = unicode(chap, 'iso-8859-1').encode('utf-8');

        fd = urllib.urlopen(wiki_url+"Chapter%sEn" % num, urllib.urlencode(forminfo));
        res = fd.read();
        fd.close();
